#!/usr/local/bin/perl

#
# NeroHistAnalyzer v1.0 (c) 2008 by Gillen Daniel <Gillen.Dan@pinguin.lu>
#
# Nero Burning Rom NeroHistory.log analyzer
#
# Inspired by nero-log.pl by Control <Control@hack.gr>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#   1. Redistributions of source code must retain the above copyright notice,
#      this list of conditions and the following disclaimer.
#   2. Redistributions in binary form must reproduce the above copyright notice,
#      this list of conditions and the following disclaimer in the documentation
#      and/or other materials provided with the distribution.
#   3. All advertising materials mentioning features or use of this software
#      must display the following acknowledgement:
#
#      This product includes software developed by Gillen Daniel.
#
#   4. Neither my name nor the names of any contributors may be used to endorse
#      or promote products derived from this software without specific prior
#      written permission.
#
# THIS SOFTWARE IS PROVIDED AS IS AND ANY EXPRESS OR IMPLIED WARRANTIES,
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDER OR ANY CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

if(@ARGV==0 || @ARGV!=4) {
  &print_usage;
} else {
  if($ARGV[0] eq "-i" && $ARGV[2] eq "-o") {
    &process_history;
  } else {
    &print_usage;
  }
}

sub print_usage {
  print "Nero Burning Rom history analyzer (c) 2008 by Gillen Daniel\n\n";
  print "Usage :\n";
  die "  perl $0 -i <input filename> -o <output filename>\n";
}

sub process_history {
  open(HISTORY, $ARGV[1]) or die "ERROR: Can't open file $ARGV[1] for reading!\n";
  open(RESULTS, ">$ARGV[3]") or die "ERROR: Can't open file $ARGV[3] for writing!\n";
  print RESULTS "Nero Burning Rom history analyzer v1.0 (c) 2008 by Gillen Daniel\n\n";
  # Extract Nero version
  while($line=<HISTORY>) {
    if($line =~ /^.*[v,V]ersion: (.*)$/m) {
      print RESULTS "Nero version: $1\n\n";
      last;
    }
  }
  # Extract CD-Rom device map
  while($line=<HISTORY>) { last if($line =~ /^=== CDRom-Device-Map ===$/m); }
  print RESULTS "CD-Rom device map:\n";
  while($line=<HISTORY>) {
    if($line !~ /^=======================$/m) {
      $line =~ /^(.*)$/m;
      print RESULTS "  * $1\n";
    } else {
      last;
    }
  }
  print RESULTS "\n";
  # Extract burned CDs / DVDs
  $line =~ /^(.*)$/m;
  $last_line=$1;
  print RESULTS "List of burned CDs / DVDs:\n";
  while($line=<HISTORY>) {
    if($line =~ /^([0-9]{1,4}\.[0-9]{1,2}\.[0-9]{1,4})$/m) {
      $1 =~ /^([0-9]{1,4})\.([0-9]{1,2})\.([0-9]{1,4})$/;
      if($1<9) { $tmp1="0$1"; } else { $tmp1=$1; }
      if($2<9) { $tmp2="0$2"; } else { $tmp2=$2; }
      if($3<9) { $tmp3="0$3"; } else { $tmp3=$3; }
      print RESULTS "  - $tmp1.$tmp2.$tmp3 ";
      # Next line contains job info
      $line=<HISTORY>;
      $line =~ /^(.*)$/m;
      $tmp=$1;
      # Next line contains start time of burn job
      $line=<HISTORY>;
      $line =~ /^([0-9]{1,2}:[0-9]{1,2}:[0-9]{1,2}).*$/m;
      print RESULTS "$1 $tmp\n";
      # Try to fetch more details of burn job
      $tracks="";
      $errors="";
      while($line=<HISTORY>) {
        if($line =~ /^====== \*\*\* Nero Burning ROM, History File \*\*\* ======$/m) {
          # After this line starts a new job, so end searching for details
          last;
        } elsif($line =~ /^.*(Recorder:.*), (Media type:.*)$/m) {
          print RESULTS "    -> $1\n";
          print RESULTS "    -> $2\n";
        } elsif($line =~ /^.*(Recorder:.*);$/m) {
          print RESULTS "    -> $1\n";
        } elsif($line =~ /^.*(Disc Manufacturer:.*)$/m) {
          print RESULTS "    -> $1\n";
        } elsif($line =~ /^.*(Track [0-9]{1,2}: Length: [0-9]{2}:[0-9]{2}\.[0-9]{2}).*(Name: .*)$/m) {
          $tracks="$tracks    * $1 $2\n";
        } elsif($line =~ /^(.*[a,A]borted.*)$/m) {
          $errors="$errors    -> ABORTED! ($1)\n";
        } elsif($line =~ /^(.*[f,F]ailed.*)$/m) {
          if($line !~ /^.*CD type reading failed.*$/m) {
            $errors="$errors    -> FAILED! ($1)\n";
          }
        }
      }
      if($errors ne "") {
        print RESULTS $errors;
      }
      if($tracks ne "") {
        print RESULTS $tracks;
      }
    }
    $line =~ /^(.*)$/m;
    $last_line=$1;
  }
  close(HISTORY);
  close(RESULTS);
}